<?php namespace awp;
class DBTable
{
	public function __construct($tableName, $fields)
	{

		$exampleFields = array(
			(object)["name" => "", "type" => "", "length"=> 255, "nullable"=>true, "unique"=>false, "default"=>""],
		);

		$this->name = sanitize_key($tableName);
		//add_action("init", function() use($fields)
		{
			$this->dataTypesValidators = apply_filters("DataTypesValidators", $this->DefaultTypeValidators());
			$this->fields = apply_filters("ValidateFormStructure",$fields);
		}//);

		add_filter("ValidateFormStructure", function($input){ return $this->DefaultStructureVlaidator($input); } );
	}

	private function DefaultTypeValidators($existingValidators = array())
	{


		//why 3,
		//https://stackoverflow.com/questions/10081500/mysql-char-varchar-character-sets-storage-sizes#:~:text=For%20ASCII%2C%20that's%201%20byte,and%20VARCHAR%20are%20the%20same.

		$charBytes = str_contains( strtolower(DB_CHARSET), "utf8") ? 3 : 1;/*askii*/;
		if(!defined("DB_CHAR_BYTES"))
		define("DB_CHAR_BYTES", $charBytes);

		//type descriptions
		//https://www.w3schools.com/mysql/mysql_datatypes.asp#:~:text=In%20MySQL%20there%20are%20three,numeric%2C%20and%20date%20and%20time.

		$newValidators = (object)[
			"char" => function( $fieldData, $input ){ 
				return $this->LengthInputCheck($fieldData, $input) && $this->FixedLengthCheck(255, $input); 
			},
			"varchar" => function( $fieldData, $input ){ 
				return $this->LengthInputCheck($fieldData, $input) && $this->FixedLengthCheck(65535, $input); 
			},
			"binary" => function( $fieldData, $input ){
				return $this->LengthInputCheck($fieldData, $input) && $this->FixedLengthCheck(255/DB_CHAR_BYTES, $input); 
			},
			"varbinary" => function( $fieldData, $input ){ 
				return $this->LengthInputCheck($fieldData, $input) && $this->FixedLengthCheck(65535/DB_CHAR_BYTES, $input); 
			},
			"tinyblob" => function( $fieldData, $input ){ 
				return $this->FixedLengthCheck(255/DB_CHAR_BYTES, $input); 
			},
			"tinytext" => function( $fieldData, $input ){ 
				return $this->FixedLengthCheck(DB_CHAR_BYTES, $input); 
			},
			"text" => function( $fieldData, $input ){ 
				return (isset($fieldData->length) ? $this->LengthInputCheck($fieldData, $input) : true) && $this->FixedLengthCheck(65535/DB_CHAR_BYTES, $input); 
			},
			"blob" => function( $fieldData, $input ){ 
				return $this->LengthInputCheck($fieldData, $input) && $this->FixedLengthCheck(65535/DB_CHAR_BYTES, $input); 
			},
			"enum" => function( $fieldData, $input ){ 
				return $this->EnumInputCheck($fieldData, $input); 
			},
			"set" => function( $fieldData, $input ){ 
				return $this->SetInputCheck($fieldData, $input) && $this->FixedLengthCheck(65535/DB_CHAR_BYTES, $input); 
			},
			"bit" => function( $fieldData, $input ){ 
				return $this->FixedIntInputCheck(64, $input); 
			},
			"tinyint" => function( $fieldData, $input ){ 
				return $this->FixedIntInputCheck(255, $input); 
			},
			"bool" => function( $fieldData, $input ){ 
				return is_bool($input); 
			},
			"boolean" => function( $fieldData, $input ){ 
				return is_bool($input); 
			},
			"smallint" => function( $fieldData, $input ){ 
				return $this->FixedIntInputCheck(65535, $input); 
			},
			"int" => function( $fieldData, $input ){ 
				return $this->FixedIntInputCheck(4294967295, $input); 
			},
			"integer" => function( $fieldData, $input ){ 
				return $this->FixedIntInputCheck(4294967295, $input); 
			},
			"float" => function( $fieldData, $input ){ 
				return is_float($input); 
			},
			"double" => function( $fieldData, $input ){ 
				return is_double($input); 
			},
			"decimal" => function( $fieldData, $input ){ 
				return is_float($input); 
			},
			"date" => function( $fieldData, $input ){ 
				return preg_match("/[1-9][0-9]{3}\-(?:0[1-9]|1[0-2])\-(?:0[1-9]|[12][0-9]|3[01])/", $input); 
			},
			"datetime" => function( $fieldData, $input ){ 
				return preg_match("/[1-9][0-9]{3}\-(?:0[1-9]|1[0-2])\-(?:0[1-9]|[12][0-9]|3[01]) (?:[01][0-9]|2[0-3])(?:\:[0-5][0-9]){2}/", $input); 
			},
			"timestamp" => function( $fieldData, $input ){ 
				return true;/*to complex*/ 
			},
			"time" => function( $fieldData, $input ){ 
				return preg_match("/\-?(?:(?:8[0-3][0-8])|[0-7]?\d?\d)(?:\:[0-5][0-9]){2}/", $input); 
			},
			"year" => function( $fieldData, $input ){ 
				return preg_match("/0{4}|190[1-9]|19[1-9][0-9]|20[0-9]{2}|21[0-5]{2}/", $input); 
			},
		];

		return (object) array_merge((array)$existingValidators, (array)$newValidators);
	}

	private function GetField($name)
	{
		for($i=0; $i<count($this->fields); $i++){
			if($this->fields[$i]->name == $name)
				return $this->fields[$i];
		}

		echo "Cannot Find field with name: {$name}.<br/>";
		debug_print_backtrace();
		die();
	}

	private function FixedIntInputCheck($size, $input)
	{
		if(is_int($input) &&  $input <= $size)
			return true;
	}

	private function EnumInputCheck($fieldData, $input)
	{
		if(is_string($fieldData))
			$fieldData = $this->GetField($fieldData);

		if(is_array($fieldData->length) && in_array($input, $fieldData->length))
			return true;

	}

	private function SetInputCheck($fieldData, $input)
	{
		if(is_string($fieldData))
			$fieldData = $this->GetField($fieldData);

		if(is_string($input))
			$input = explode(",", $input);

		if(!is_array($input) || !is_array($fieldData->length) || count($input) > 64)
			return false;

		for($i=0; $i<count($input); $i++)
		{
			if(!in_array($input[$i], fieldData->length))
			{
				return false;
			}
		}

		return true;
	}

	private function LengthInputCheck($fieldData, $input)
	{

		if(is_string($fieldData))
			$fieldData = $this->GetField($fieldData);

		if(is_string($input) && strlen($input) <= $fieldData->length)
			return true;
	}

	private function FixedLengthCheck($length, $input)
	{
		if(is_string($input) && strlen($input) <= $length)
			return true;
	}

	private function DefaultStructureVlaidator($input)
	{
		for($i=0; $i<count($input); $i++)
		{
			$fieldData = $input[$i];

			if(!property_exists($fieldData, "name") || !property_exists($fieldData, "type") )
			{
				echo "Field Data Invalid: must contain name and type properties.";
				var_dump($fieldData);
				die();
			}

			$fieldKeys = array_keys(get_object_vars($fieldData));
			for($j=0; $j<count($fieldKeys); $j++)
			{
				$property = $fieldKeys[$j];
				$value = $fieldData->$property;


				switch($property){
					case "name":
						$this->ValidateNameStructure($value);
						break;
					case "type":
						$this->ValidateTypeStructure(strtolower($value));
						break;
					case "length":
						//$this->ValidteLenghtStructure($value);
						//ahh can be array to
						break;
					case "nullable":
					case "unique":
					case "primary":
					case "primaryKey":
						$this->ValidateBoolStructure($value);
						break;
					case "default":
						$this->ValidateDefaultStructure($fieldData, $value);
						break;
				}


			}

		}

		return $input;
	}

	private function ValidateNameStructure($value)
	{
		if($value != sanitize_key($value))
		{
			echo "Field name {$value} is invalid.<br /> you can try this: " . sanitize_key($value);
			die();
		}

		return true;
	}

	private function ValidateTypeStructure($value)
	{

		if(!property_exists($this->dataTypesValidators, $value))
		{
			echo "No validation methods for field type: {$value}.<br/> Available Validators: ";
			var_dump(array_keys(get_object_vars($this->dataTypesValidators)));
			die();
		}

		return true;
	}

	private function ValidateLengthStructure($value)
	{
		if(intval($value) <= 0)
		{
			echo "Field length must be an int more than 0";
			die();
		}

		return true;
	}

	private function ValidateBoolStructure($value, $errorMessage = null)
	{
		if(! is_bool($value))
		{
			echo $errorMessage || "value must be a bool";
			die();
		}

		return true;
	}

	private function ValidateDefaultStructure($fieldData, $value)
	{


		if( isset($fieldData->ignoreDefaultCheck) && $fieldData->ignoreDefaultCheck)
			return true;

		$type = $fieldData->type;
		$callable = $this->dataTypesValidators->$type;

		if(!$callable($fieldData, $value))
		{
			echo "Default value failed validation check";
			die();
		}

		return true;
	}

	public function TableName()
	{
		global $wpdb;
		return $wpdb->prefix.$this->name;
	}

	public function CrateTable()
	{
		global $wpdb;

		$charset = $wpdb->get_charset_collate();
		$tableName = $wpdb->prefix . $this->name;

		//if($this->TableExists($tableName)) return;

		$rows = array();

		$sql = "CREATE TABLE {$tableName} (";
		$primaryKeyName = null;

		for($i=0; $i<count($this->fields); $i++)
		{
			$field = $this->fields[$i];
			$name = ($field->name);
			$length = property_exists($field, "length") ? $field->length : null;


			if(is_array($length))
			{
				$length = "";

				for($j=0; $j<count($field->length); $j++)
				{
					$item = $field->length[$j];
					if($j != 0)
						$length .= ", ";
					$length .= is_numeric($item) ? $item : "'{$item}'";
				}
			}

			$type = $field->type . (!empty($length) ? "({$length})":"" );
			$default = isset($field->default) && !empty($field->default) ? " DEFAULT '" . $field->default . "'" : "";
			$nullable =  isset($field->nullable) && !$field->nullable ? " NOT NULL" : "";
			$autoIncrament = isset($field->autoIncrament) && $field->autoIncrament ? " AUTO_INCREMENT " : "";
			$unique = isset($field->unique) && $field->unique ? " UNIQUE " : " ";
			//$primaryKey = isset($field->primaryKey) && $field->primaryKey ? " PRIMARY KEY" : "";
			//$primaryKey = isset($field->primary) && $field->primary ? " PRIMARY KEY" : $primaryKey;

			$pkTrue = (isset($field->primaryKey) && $field->primaryKey) || (isset($field->primary) && $field->primary);
			
			$primaryKeyName = empty($primaryKeyName) && $pkTrue ? $name : $primaryKeyName;

			$sql .=  ($i!=0?",":"") . "\n\t{$name} {$type}". $default . $nullable . $autoIncrament . $unique /*. $primaryKey*/;
		}
		$sql .= "\n\tPRIMARY KEY (`{$primaryKeyName}`)";
		
		$sql .="\n) {$charset};";

		//print_r($sql);
		//die();

		require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
		dbDelta( $sql );
	}

	private function TableExists($tableName)
	{
		global $wpdb;

		$query = $wpdb->prepare( 
			"SHOW TABLES LIKE '%s';", 
			\esc_sql( $tableName ) 
		);

		if ( $wpdb->get_var( $query ) == $tableName ) {
		    return true;
		}
	}

	public function DeleteTable()
	{
		global $wpdb;

		$tableName = $wpdb->prefix . $this->name;
		if(!$this->TableExists($tableName)) return;

	    $sql = "DROP TABLE IF EXISTS {$tableName}";
	    $wpdb->query($sql);
	    
	    //delete_option('option name');
	}

	public function UpdateRow($rowData, $where, $dataFormat = [null], $whereFormat =[null])
	{

		global $wpdb;
	    $tableName = $wpdb->prefix.$this->name;
	    $queryData = $this->GetWhereString($where);

	    if(empty($dataFormat))
	    {
	    	$dataKeys = array_keys($rowData);
	    	for($i=0; $i<count($dataKeys); $i++)
	    	{
	    		$dataFormat[] = "%s";
	    	}
	    }

	    if(empty($whereFormat))
	    {
	    	$whereKeys = array_keys($where);
	    	for($i=0; $i<count($whereKeys); $i++)
	    	{
	    		$whereFormat[] = "%s";
	    	}
	    }

		$r = $wpdb->update( $tableName, (array)$rowData, (array)$queryData->labeled /*, $dataFormat, $whereFormat*/ );

     	return $r;
	}

	public function GetRows($where, $select = array(), $order = null, $limmit = null)
	{
		global $wpdb;
	    $tableName = $wpdb->prefix.$this->name;

	    ini_set('display_errors', 1);
		ini_set('display_startup_errors', 1);
		error_reporting(E_ALL);

		$selectString = "";
		if(empty($select))
		{
			$selectString = "*";
		}
		elseif(is_array($select) && array_keys($select) === range(0, count($select) - 1))
		{
			for($i=0; $i<count($select); $i++)
			{
				$selectString .= $i > 0 ? ", " : " ";
				$selectString .=  $select[$i];
			}
		}
		elseif(is_array($select)) //associative array
		{
			$selectKeys = array_keys($select);
			for($i=0; $i<count($selectKeys); $i++)
			{
				$selectString .= $i > 0 ? ", " : " ";
				$selectString .=  $selectKeys[$i] . ($select[$selectKeys[$i]] != $selectKeys[$i] ? " AS {$select[$selectKeys[$i]]}" : "");
			}
		}
		else // object
		{
			$selectKeys = array_keys(get_object_vars($select));
			for($i=0; $i<count($selectKeys); $i++)
			{
				$key = $selectKeys[$i];
				$selectString .= $i > 0 ? ", " : " ";
				$selectString .=  $selectKeys[$i] . ($select->$key != $selectKeys[$i]? " AS {$select->$key}" : "");
			}
		}

	    $queryDataP = $this->GetWhereString($where);


	    if($order !== null)
	    {
	    	$queryDataP->string .= " ORDER BY {$order}";
	    }

	    if(is_int($limmit) && $limmit !== null)
	    {
	    	$queryDataP->string .= " LIMIT {$limmit}";
	    }

	    $queryData = array("SELECT {$selectString} FROM $tableName ".$queryDataP->string, $queryDataP->values);
	    
	    $query = "";
	    if(empty($queryDataP->values))
	    {
	    	$query = $queryData[0];
	    }
	    else
	    {
	    	$query =call_user_func_array([$wpdb, "prepare"], $queryData);
	    }

		$data = $wpdb->get_results($query);


		return $data;
	}

	public function InsertRow($rowData)
	{
		global $wpdb;
	    $tableName = $wpdb->prefix.$this->name;


		for($i=0; $i<count($this->fields); $i++)
		{
			$tableData = $this->fields[$i];
			if(property_exists($rowData, $tableData->name))
			{
				$rowName = $tableData->name;
				if(!$this->ValidateInput($tableData, $rowData->$rowName))
				{
					echo "Failed to validate {$tableData->name}.";
					die();
					return false;
				}
			}
			elseif(isset($tableData->required) && $tableData->required)
			{
				echo "{$tableData->name} is required.";
				die();
				return false;
			}
		}


     	return $wpdb->insert( $tableName, (array)$rowData);
	}

	public function DeleteRow($where)
	{
		global $wpdb;
	    $tableName = $wpdb->prefix.$this->name;

	    $queryData = $this->GetWhereString($where);

	    return $wpdb->delete( $tableName, $queryData->labeled, $queryData->markers);

	}

	private function GetWhereString($input)
	{
		$example = array((object)["key"=>"id", "modifier"=>"=", "value"=>null, "marker"=>"%s" ]);

	    $whereString = "";
	    $whereValues = array();
	    $whereLabeled = array(); //FOR DELETE FUNCTION
	    $whereMarkers = array();//FOR DELETE BUT MAY BE USEFUL

	    if(!is_array($input) || $this->IsAssoc($input))
	    {
	    	$input = array($input);
	    }

	    $inputKeys = array_keys($input);
	    for ($i=0; $i<count($inputKeys); $i++)
	    {
	    	$queryItem = (object)$input[$inputKeys[$i]];
	    	$key = isset($queryItem->key)? $queryItem->key : "id";
	    	$modifier = isset($queryItem->modifier)? $queryItem->modifier : "=";
	    	$marker = isset($queryItem->marker)? $queryItem->marker : "%s";
	    	$value = isset($queryItem->value)? $queryItem->value : null;

	    	if($i ==0) 
	    		$whereString = "WHERE ";
	    	else 
	    		$whereString .= " AND ";

	    	if(isset($queryItem->value))
	    	{
	    		$whereString .= "{$key} ". strtoupper($modifier) . " '{$marker}'";
	    		$whereValues[] = $value;
	    		$whereLabeled[$key] = $value;
	    		$whereMarkers[] = $marker;
	    	}
	    }

	    return (object)[
	    	"string" => $whereString, 
	    	"values" => $whereValues, 
	    	"labeled" => $whereLabeled, 
	    	"markers" => $whereMarkers
	    ];
	}

	private function IsAssoc($arr)
	{
	    if (array() === $arr) return false;
	    return array_keys($arr) !== range(0, count($arr) - 1);
	}

	private function ValidateInput($fieldData, $input)
	{

		$type = $fieldData->type;
		if($this->ValidateTypeStructure($type))
		{
			$validator = $this->dataTypesValidators->$type;
			return $validator($fieldData, $input);
		}
	}

	public static function ByteString($len = 16)
	{

		return openssl_random_pseudo_bytes($len);
	}

	public static function HexString($byteString = 8)
	{
		$data = $byteString;

		if(is_int($byteString))
			$data = openssl_random_pseudo_bytes($byteString);


		return rtrim(base64_encode($data), "=");
	}

	public static function HexToByte($hexString, $len=8)
	{
		$hex = str_pad($hexString, $len, "=");
		return base64_decode($hex);
	}

	//https://stackoverflow.com/questions/2040240/php-function-to-generate-v4-uuid
	public static function UUID($byteString = 16)
	{
		$data = $byteString;

		if(is_int($byteString))
			$data = openssl_random_pseudo_bytes($byteString);

		if(strlen($data) != 16)
			return null;

    	$data[6] = chr(ord($data[6]) & 0x0f | 0x40); // set version to 0100
    	$data[8] = chr(ord($data[8]) & 0x3f | 0x80); // set bits 6-7 to 10

    	return vsprintf("%s%s-%s-%s-%s-%s%s%s", str_split(bin2hex($data), 4));
	}

}

?>