<?php namespace awp;

class Debug extends Module
{

	public function __construct()
	{ 
		//debug functions
		add_action( "WriteEnqued", array($this, "WriteEnqued") );
		add_action( "ConsolePageData", array($this, "ConsolePageData") );
		add_action( "HideAdminMenuBackgroud", array($this, "HideAdminMenuBackgroud") );
		
		$this->InitAdminMessage();
		
	}
	
	private function InitAdminMessage()
	{
		if (!session_id()) {
		    session_start();
		}
		
		if(!isset($_SESSION["debugMsg"]) || !is_array($_SESSION["debugMsg"]))
		{
			return;	
		}
		
		for($i=0; $i<count($_SESSION["debugMsg"]); $i++)
		{
			call_user_func_array(array($this, "AdminMessage"), $_SESSION["debugMsg"][$i]);
		}
		
		unset($_SESSION["debugMsg"]);
		
	}
	
	
	public function AdminMessage($string, $type="info", $dismissable = false)
	{
		add_action( 'admin_notices', function() use($string, $type, $dismissable){
			$class = "notice notice-{$type}" . ($dismissable ? " is-dismissible" : "");
			printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $string ) );
		});
		
		add_action( 'admin_store_notices', function() use($string, $type, $dismissable){
			
			if (!session_id()) 
			    session_start();
			
			if(!isset($_SESSION["debugMsg"]) || !is_array($_SESSION["debugMsg"]))
				$_SESSION["debugMsg"] = array();
				
			
			$_SESSION["debugMsg"][] = array($string, $type, $dismissable);
			
			//var_dump($_SESSION["debugMsg"]);die();
			
		});
	}
	
	public function HideAdminMenuBackgroud()
	{
		add_action( "wp_head", function()
		{ 
			?><!-- HIDE MENU BACKGROUND --><style type="text/css">#adminmenuback {  display: none;} </style><?php
		} );
	}
	   

	public function WriteEnqued() 
	{
		add_action( 'wp_footer', function(){  print_r($this->EnquedItems()); });

	}
	
	public function ConsolePageData()
	{
		add_action( 'site_enqueue_scripts', function()
		{
			$debugHandle = "debug_page_data";
			global $template, $wp_query;
			$t = $template;
	
			$content = [
				"pageName" => isset($wp_query->query_vars["name"]) ? $wp_query->query_vars["name"] : "",
				"pageType" => $this->PageType(),
				"template" => @end(explode("wp-content/", $template)),
				"styles" => $this->EnquedItems()['styles'],
				"scripts" => $this->EnquedItems()['scripts'],
				"postType" => get_post_type(),
				"postName" => get_post_field( 'post_name', get_post()),
			];
	
			wp_register_script( $debugHandle, '');
			wp_enqueue_script($debugHandle);
			wp_localize_script( $debugHandle, $debugHandle,  ($content));
			wp_add_inline_script($debugHandle, "console.log({$debugHandle});");

		} , 20 );
	}

	private function EnquedItems()
	{
		global $post;
		$result = [];
		$result['scripts'] = [];
		$result['styles'] = [];

		// Print all loaded Scripts
		global $wp_scripts;
		foreach( $wp_scripts->queue as $script ) :
		$result['scripts'][] =  $wp_scripts->registered[$script]->handle . ";";
		endforeach;

		// Print all loaded Styles (CSS)
		global $wp_styles;
		foreach( $wp_styles->queue as $style ) :
		$result['styles'][] =  $wp_styles->registered[$style]->handle . ";";
		endforeach;

		return $result;
	}

	private function PageType() {
		global $wp_query;
		$loop = 'notfound';

		if ( $wp_query->is_page ) {
			$loop = is_front_page() ? 'front' : 'page';
		} elseif ( $wp_query->is_home ) {
			$loop = 'home';
		} elseif ( $wp_query->is_single ) {
			$loop = ( $wp_query->is_attachment ) ? 'attachment' : 'single';
		} elseif ( $wp_query->is_category ) {
			$loop = 'category';
		} elseif ( $wp_query->is_tag ) {
			$loop = 'tag';
		} elseif ( $wp_query->is_tax ) {
			$loop = 'tax';
		} elseif ( $wp_query->is_archive ) {
			if ( $wp_query->is_day ) {
				$loop = 'day';
			} elseif ( $wp_query->is_month ) {
				$loop = 'month';
			} elseif ( $wp_query->is_year ) {
				$loop = 'year';
			} elseif ( $wp_query->is_author ) {
				$loop = 'author';
			} else {
				$loop = 'archive';
			}
		} elseif ( $wp_query->is_search ) {
			$loop = 'search';
		} elseif ( $wp_query->is_404 ) {
			$loop = 'notfound 404';
		}

		return $loop;
	}
	
	
	
}
?>