<?php namespace awp;
class NewPluginPage extends ConfigPage
{
	public function __construct($parent)
	{
        
		$this->app = \awp\App::Instance();
		
        parent::__construct(
            array("parent"=> $parent, "slug"=>"ash-wp-new-plugin", "title"=> "Create Plugin", "tabTitle"=>false), 
            array("style-forms", "style-buttons", "style-cards")
        );
        
		
	}
	
	
	/**
	 * Called to initialise data before page loads, usually setup database and fetch field data. Is called before both load and request.
	 **/ 
	protected function Begin()
	{
		
		
        
		$newPluginFields = [
			"name" => ["label"=>"Name", "default"=>"", "type"=>"text", "description"=>""],
			"description" => ["label"=>"Description", "default"=>"", "type"=>"textarea", "description"=>""],
			"author" => ["label"=>"Author Name", "default"=>"", "type"=>"text", "description"=>""],
			"authorUrl" => ["label"=>"Author URL", "default"=>"", "type"=>"text", "description"=>""],
			"slug" => ["label"=>"Slug", "default"=>"", "type"=>"text", "description"=>""],
			
			
        ];
        
		$formData = array(
			"ash-wp-new-plugin-form" => array(
				"ash-wp-plugin-options" => array_merge(
					array("groupTitle" => "New Plugin"),
					$newPluginFields,
				),
				"submit" => array(
					"text" => "",
					"action" => "create-new-plugin",
					"settingName" => "ash-wp-new-plugin",
				)
			)
		);
		
		$this->AddComponent($formData);
		
	}

	/**
	 * Called before render page
	 **/ 
	protected function Load(){}

	/**
	 * Called when data is posted.
	 **/ 
	protected function Request($data)
	{
		
		if(!$this->RequreKeys($data, "slug", "name"))
		{
			$this->app->AdminMessage("slug and name reqired.", "error");
		}
		
		
		if(strlen($data->slug) <= "3" || sanitize_title($data->slug) != $data->slug)
		{
			$this->app->AdminMessage("slug must be url safe and  >3 chars", "error");
			$this->RedirectBack();
		}

		$pluginsFolder = ABSPATH . "wp-content/plugins/";
		$pluginFolder = $pluginsFolder . $data->slug . "/";
		
		if (!file_exists($pluginFolder)) 
		{
			mkdir($pluginFolder, 0755, true);
			mkdir($pluginFolder."pages/", 0755, true);
			mkdir($pluginFolder."data/", 0755, true);
		}
		else
		{
			//could not create plugin, plugin folder already exists.	
			
			$this->app->AdminMessage("Could not create plugin, plugin folder already exists.", "error");
			$this->RedirectBack();
		}
		
		$pluginEntryPoint = file_get_contents($pluginsFolder . "ash-wp/templates/plugin.php");
		$pluginEntryPoint = str_replace("{{name}}", $data->name, $pluginEntryPoint);
		$pluginEntryPoint = str_replace("{{slug}}", $data->slug, $pluginEntryPoint);
		$pluginEntryPoint = str_replace("{{description}}", isset($data->description) ? $data->description : "", $pluginEntryPoint);
		$pluginEntryPoint = str_replace("{{author}}", isset($data->author) ? $data->author : "", $pluginEntryPoint);
		$pluginEntryPoint = str_replace("{{authorUrl}}", isset($data->authorUrl) ? $data->authorUrl : "", $pluginEntryPoint);
		
		
		file_put_contents($pluginFolder . $data->slug . ".php", $pluginEntryPoint);
		
		$this->app->AdminMessage("Plugin Created", "success");
		$this->RedirectBack();
		
		//todo:
		//copy files to new folder
		//generate config in plugin-slug.php
	}

}
?>